@@ -117,6 +117,10 @@ class Node:
         initialize_ops_grad(self)
         return outputs
 
+    # 矩阵乘
+    def __matmul__(self, other):
+        return self.matmul(other)
+
     def backward(self, gradients: GlobalGraph.np.ndarray = None):
         if self.grad_fn is None:
             raise ValueError('can not solve grad on %s' % self)
@@ -131,7 +135,7 @@ class Node:
             else:
                 raise ValueError('grad can be implicitly created only for scalar outputs')
 
-        self.grad_fn(self)
+        # self.grad_fn(self)
         if GlobalGraph.outputs is None:
             GlobalGraph.outputs = self
         if GlobalGraph.inputs is not None:
@@ -176,6 +180,9 @@ class Node:
         other.out_bounds.append(outputs)
         return outputs
 
+    def dot(self, other):
+        return self.matmul(other)
+
     # 转置
     def t(self):
         if GlobalGraph.inputs is None:
@@ -212,8 +219,8 @@ class Node:
             mean_value = np.mean(self.data, keepdims=keepdims)
         else:
             mean_value = np.mean(self.data, axis=axis, keepdims=keepdims)
-        self.cache['axis'] = axis
         outputs = Variable(in_bounds=[self], data=mean_value)
+        outputs.cache['axis'] = axis
         # 绑定反向求梯度的函数
         outputs.grad_fn = MeanBackward
         initialize_ops_grad(self)
@@ -285,6 +292,9 @@ class Node:
         outputs = Variable(in_bounds=[self], data=np.argmax(self.data, axis=axis))
         return outputs
 
+    def numel(self):
+        return self.data.size
+
 
 class Variable(Node):
     def __init__(self, data: Union[GlobalGraph.np.ndarray, int, float], in_bounds: List = None,
@@ -303,12 +313,10 @@ class Variable(Node):
                       requires_grad=requires_grad)
 
     def __getitem__(self, item):
-        ret = Variable(data=self.data[item])
-        return ret
+        return slices(self, item)
 
     def __setitem__(self, key, value):
-        self.data[key] = value.data
-        self.in_bounds.append(value)
+        copySlices(self, key, value)
 
 
 class Constant(Node):
@@ -371,6 +379,9 @@ class Layer:
         inbound.out_bounds.append(self)
         return self
 
+    def __rshift__(self, other):
+        return other.__call__(self)
+
     def connect(self, inbound=None):
         # inbound只能是一个Layer
         if inbound is None:
@@ -381,7 +392,6 @@ class Layer:
         self.shape = self.compute_output_shape(self.input_shape)
         if inbound is not None:
             self.in_bounds.append(inbound)
-            self.input_shape = inbound.shape
             inbound.out_bounds.append(self)
 
     # 初始化本层参数，由子类单独实现
@@ -417,6 +427,7 @@ class Layer:
         for inbound in self.in_bounds:
             if inbound.data.requires_grad:
                 inbound.data.grad += gradients
+        # raise NotImplementedError
 
 
 # 加法
@@ -462,17 +473,46 @@ def mul(x: Variable, y: Variable) -> Variable:
 
     # 列举所有做点乘的情况, 1.两个标量， 2.两个矩阵的shape一样。
     # 有一种特殊情况，两个矩阵的shape虽然不一样，但还是可以通过broadcast机制做点乘，比如(1, 3) * (3, 3) = (3, 3)，但是也可以理解为矩阵乘 = (1, 3)，这种情况下，我们默认为矩阵乘。
-
-    if len(x.shape) == 1 and len(y.shape) == 1 or x.shape[-1] != y.shape[0]:
+    if isinstance(y, int) or isinstance(y, float):
+        y = Variable(y, requires_grad=False)
+    outputs = None
+    if len(x.shape) == 1 or len(y.shape) == 1:
+        # 点乘
+        outputs = Variable(in_bounds=[x, y], data=x.data * y.data, requires_grad=True)
+        outputs.grad_fn = MultiplyBackward
+    elif len(x.shape) == len(y.shape):
+        for s1, s2 in zip(x.shape, y.shape):
+            if s1 != s2 and s1 != 1 and s2 != 1:
+                # 矩阵乘
+                outputs = Variable(in_bounds=[x, y], data=np.dot(x.data, y.data), requires_grad=True)
+                outputs.grad_fn = MatmulBackward
+                break
         # 点乘
         outputs = Variable(in_bounds=[x, y], data=x.data * y.data, requires_grad=True)
         outputs.grad_fn = MultiplyBackward
     else:
-        # 矩阵乘
-        outputs = Variable(in_bounds=[x, y], data=np.dot(x.data, y.data), requires_grad=True)
-        outputs.grad_fn = MatmulBackward
-
+        raise ValueError('can\'t peroform either multiply nor matmul between {} and {}'.format(x, y))
     initialize_ops_grad(x, y)
     x.out_bounds.append(outputs)
     y.out_bounds.append(outputs)
     return outputs
+
+
+def slices(x, item):
+    if GlobalGraph.inputs is None:
+        GlobalGraph.inputs = x
+    ret = Variable(data=x.data[item], in_bounds=[x, ])
+    x.out_bounds.append(ret)
+    ret.cache['pos'] = item
+    initialize_ops_grad(x)
+    ret.grad_fn = SlicesBackward
+    return ret
+
+
+def copySlices(x, pos, value: Variable):
+    x.data[pos] = value.data
+    value.out_bounds.append(x)
+    x.in_bounds.append(value)
+    x.cache[pos] = value
+    initialize_ops_grad(value)
+    x.grad_fn = CopySlicesBackward
